/* gxa2d.c Copyright 2005 by WinSystems Inc. All Rights Reserved
**********************************************************************
*
*   Name    : gxa2d.c
*
*   Project : EPX-GX User interface code
*
*   Purpose : Analog input routines
*
*   Date    : September 7, 2005
*
*   Revision: 1.00
*
*   Author  : Steve Mottin
*
**********************************************************************
*
*   Revision History
*
*   Date        Revision    Description
*   --------    --------    ----------------------------------------
*   09/07/05    1.00        Initial Version
*
**********************************************************************
*/

#include <stdio.h>
#include <dos.h>        /* For Intrinsic inportb/outportb functions */
#include <conio.h>
#include <stdlib.h>


/* The Port definitions for the registers */

#define GXAD_BASE_ADDRESS 0x1e8
#define GXAD_COMMAND_PORT GXAD_BASE_ADDRESS
#define GXAD_STATUS_PORT  GXAD_BASE_ADDRESS
#define GXAD_LSB_DATA     GXAD_BASE_ADDRESS+1
#define GXAD_MSB_DATA     GXAD_BASE_ADDRESS+2
#define GXAD_INT_CONTROL  GXAD_BASE_ADDRESS+2

/* These bits appear in the status register */

#define GXAD_INT_BIT     0x80
#define GXAD_BUSY_BIT    0x01

/* These defines make setting the mode easy. The actual register
   bit definitions have no logical order to them
*/

#define GXAD_SINGLE_ENDED 0x80
#define GXAD_DIFFERENTIAL 0x00

#define GXAD_UNIPOLAR    0x08
#define GXAD_BIPOLAR     0x00

#define GXAD_TOP_5V      0x00
#define GXAD_TOP_10V     0x04

/* Even the channel numbers are encoded in the command in a
   strange way in the register.
*/

#define GXAD_CH0_SELECT  0x00
#define GXAD_CH1_SELECT  0x40
#define GXAD_CH2_SELECT  0x10
#define GXAD_CH3_SELECT  0x50
#define GXAD_CH4_SELECT  0x20
#define GXAD_CH5_SELECT  0x60
#define GXAD_CH6_SELECT  0x30
#define GXAD_CH7_SELECT  0x70

/* To make channel selection much, much easier (no shifting and oring)
   we build a select table array the channel number can be used as the
   index into the array and the proper bit encoding for that channel id
   the result.
*/

unsigned char gxad_channel_select[8] = { GXAD_CH0_SELECT, GXAD_CH1_SELECT, GXAD_CH2_SELECT,
                GXAD_CH3_SELECT, GXAD_CH4_SELECT, GXAD_CH5_SELECT, GXAD_CH6_SELECT, GXAD_CH7_SELECT };

/* Along the same line when selecting a channel, the mode bits are also
   pre-encoded using this array. It starts out blank and is filled in
   when making calls to set_channel_mode.
*/

unsigned char gxad_channel_mode[8] = { GXAD_CH0_SELECT, GXAD_CH1_SELECT, GXAD_CH2_SELECT,
                GXAD_CH3_SELECT, GXAD_CH4_SELECT, GXAD_CH5_SELECT, GXAD_CH6_SELECT, GXAD_CH7_SELECT };

/* These three arrays are used primarily for conversion to floating point
   voltage values. THey are also filled in for a channel during a call to
   set_channel_mode. They avoid having to bit decode command bytes to
   figure out what mode is being used and therefore the voltage value of
   each bit.
*/

float gxad_bitval[8] = {.0001, .0001, .0001, .0001, .0001, .0001, .0001, .0001};
unsigned gxad_adjust[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
float gxad_offset[8] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };

/* Here are the function prototypes for all of the functions used in this
   support library.
*/

int gxad_wait_ready(void);
int gxad_start_conversion(unsigned char channel);
int gxad_set_interrupt_mode(int my_irq);
void gxad_set_polled_mode(void);
unsigned gxad_read_conversion_data(void);
int gxad_set_channel_mode(int channel, int input_mode, int duplex, int range);
int gxad_convert_all_channels(unsigned *buffer);
int gxad_convert_single_repeated(int channel, unsigned count, unsigned *buffer);
int gxad_buffered_channel_conversions(unsigned char *input_channel_buffer,unsigned *buffer);
float gxad_get_channel_voltage(int channel);
float gxad_auto_get_channel_voltage(int channel);
void gxad_install_isr(int irq);
void gxad_uninstall_isr(int irq);
void interrupt gxad_a2d_isr(void);

/* These two variables ares use for error detection and reporting */

int gxad_error_code;
char gxad_error_string[128];

/* Here are the error code definitions */

#define GXAD_SUCCESS 0
#define GXAD_OPEN_ERROR 1
#define GXAD_TIMEOUT_ERROR 2
#define GXAD_BAD_CHANNEL_NUMBER 3
#define GXAD_BAD_MODE_NUMBER 4
#define GXAD_BAD_RANGE 5
#define GXAD_COMMAND_WRITE_FAILURE 6
#define GXAD_READ_DATA_FAILURE 7

/* A bunch of global variables are used to make processing faster and to
   flag the ISR (when in interrupt mode) as to proper handling of the
   data and whether to start up another conversion from within the ISR
   itself.
*/

/* Int_mode is the switch that controls whether the code will run interrupt
   driven or in a polled mode. When int_mode is 0, polled I/O will be used,
   when non-zero interrupt I/O will be used. Because of the extreme
   speed of the LTC185X converters, interrupt mode is about 10% slower in
   most cases due to interrupt overhead.
*/

int gxad_int_mode = 0;
int gxad_irq = 0;
int gxad_intflag = 0;
int gxad_buff_mode = 0;

/*  These two variables can be useful because when a conversion is
    started the data from the previous conversion is returned. These
    variables can be used to determine which channel's data is being
    returned by read_conversion_data wich will return the last_channel's
    data. Note that the get_voltage and auto_get_voltage autmatically do
    double reads in order to return the true currently requested channel
    voltage.
*/

int gxad_last_channel = 0xff;
int gxad_current_channel = 0xff;

/* These are used to control repeated single channel conversions at
   in a high-speed buffered manner.
*/

int gxad_repeat_mode = 0;
int gxad_repeat_count = 0;
int gxad_repeat_channel = 0;
int gxad_repeat = 0;

/* Storage for the previous contents of the specified interrupt vector.
   It is fillled in with a call to install_isr and is used by uninstall_isr
   to return the vector to its previous state.
*/

void interrupt (*old_isr)();

/* An array of channel numbers is used by the convert_all_channels function
   and the others are index pointers into user supplied buffers used
   during buffered functions.
*/

unsigned char gxad_channel_buff[10] = {1,2,3,4,5,6,7,0,0xff,0xff};
unsigned char *gxad_input_buffer;
unsigned *gxad_user_buffer;
int gxad_ch_index = 0;
int gxad_out_index;


/////////////////////////////////////////////////////////////////////////
//
//      GXAD_AUTO_GET_CHANNEL_VOLTAGE
//
//
////////////////////////////////////////////////////////////////////////
float gxad_auto_get_channel_voltage(int channel)
{
unsigned value;
float result;


		gxad_error_code = GXAD_SUCCESS;

            /* Start out on a +/- 10V scale. That covers all possible
               legal input voltages
            */

		gxad_set_channel_mode(channel,GXAD_SINGLE_ENDED,GXAD_BIPOLAR,GXAD_TOP_10V);

		if(gxad_error_code)
			return(0.0);

            /* Execute two conversions to get current values */

            gxad_start_conversion(channel);
		if(gxad_error_code)
			return(0.0);

            gxad_wait_ready();
		if(gxad_error_code)
			return(0.0);

            gxad_read_conversion_data();
		if(gxad_error_code)
			return(0.0);

            gxad_start_conversion(channel);
		if(gxad_error_code)
			return(0.0);

            gxad_wait_ready();
		if(gxad_error_code)
			return(0.0);

            /* Get the current RAW data */

            value = gxad_read_conversion_data();
		if(gxad_error_code)
			return(0.0);

            /* Convert the raw data to a voltage using our arrays */

            value = value + gxad_adjust[channel];
            result = value * gxad_bitval[channel];
            result = result +  gxad_offset[channel];

            /* If the voltage is less than -5.0 Volts. The range we used
               already is as precise as we get, just return the reading
            */

            if(result <= -5.00)  // We're done
                return(result);

            /* If the result is between -4.99 and 0.0 we can switch
               to the +/- 5 Volt scale for a more precise reading.
            */

            if(result < 0.0)
                gxad_set_channel_mode(channel,GXAD_SINGLE_ENDED,GXAD_BIPOLAR,GXAD_TOP_5V);

		if(gxad_error_code)
			return(0.0);

            /* If the value is above 5 volts a 0 - 10V range will give
               our most precise result.
            */

            if(result >= 5.00)
                gxad_set_channel_mode(channel,GXAD_SINGLE_ENDED,GXAD_UNIPOLAR,GXAD_TOP_10V);

		if(gxad_error_code)
			return(0.0);

            /* Lastly. if we're greater than zero and less than 5.0 Volts
                the 0 - 5V scale is the best choice.
            */

            if((result >= 0.0) && (result < 5.00))
                gxad_set_channel_mode(channel,GXAD_SINGLE_ENDED,GXAD_UNIPOLAR,GXAD_TOP_5V);

		if(gxad_error_code)
			return(0.0);

            /* Now that we have the value properly ranged we take two more
               samples to get a current reading at the new scale.
            */

            gxad_start_conversion(channel);
		if(gxad_error_code)
			return(0.0);

            gxad_wait_ready();
		if(gxad_error_code)
			return(0.0);

            gxad_start_conversion(channel);
		if(gxad_error_code)
			return(0.0);

            gxad_wait_ready();
		if(gxad_error_code)
			return(0.0);

            value = gxad_read_conversion_data();
		if(gxad_error_code)
			return(0.0);

            /* Convert the current data to a voltage */

            value = value + gxad_adjust[channel];
            result = value * gxad_bitval[channel];
            result = result +  gxad_offset[channel];

            /* Return the result */

            return(result);
}

//////////////////////////////////////////////////////////////////////////
//
//      GXAD_GET_CHANNEL_VOLTAGE
//
//////////////////////////////////////////////////////////////////////////
float gxad_get_channel_voltage(int channel)
{
unsigned value;
float result;

		gxad_error_code = GXAD_SUCCESS;

            /* Start two conversions os that we have current data */
            
            gxad_start_conversion(channel);
		if(gxad_error_code)
			return(0.0);

            gxad_wait_ready();
		if(gxad_error_code)
			return(0.0);

            gxad_start_conversion(channel);
		if(gxad_error_code)
			return(0.0);

            gxad_wait_ready();
		if(gxad_error_code)
			return(0.0);

            /* Read out the last conversion's raw data */

            value = gxad_read_conversion_data();
		if(gxad_error_code)
			return(0.0);

            /* Use the array values to calculate the voltage */

            value = value + gxad_adjust[channel];
            result = value * gxad_bitval[channel];
            result = result +  gxad_offset[channel];

            /* Return the result */

            return(result);
}

//////////////////////////////////////////////////////////////////////////
//
//      GXAD_BUFFERED_CHANNEL_CONVERSIONS
//
//////////////////////////////////////////////////////////////////////////
int gxad_buffered_channel_conversions(unsigned char *input_channel_buffer,unsigned *buffer)
{
int x;

    gxad_error_code = GXAD_SUCCESS;
		
    /* Transfer the addresses of the user supplied buffers to our own
       global pointers.
   */

    gxad_user_buffer = buffer;
    gxad_input_buffer = input_channel_buffer;

   /* Reset all of the array index pointers */

    gxad_ch_index = 0;
    gxad_out_index = 0;


    /* If we're running in interrupt driven mode we must start up the
    first conversion manually and the rest will be started in the Interrupt
    Service routine (ISR)
    */

    if(gxad_int_mode)
    {
        /* Tell the ISR we're in "buffered" mode */

        gxad_buff_mode = 1;

        /* Run 1 conversion just to get it started */

        gxad_start_conversion(gxad_input_buffer[gxad_ch_index++]);
        if(gxad_error_code)
		return(-1);

        /* The ISR should run all of the desired conversions due to buffered
         mode being set.
        */
    
        while(gxad_buff_mode)     /* This could lock it up good if things go bad */
        {
            if(kbhit())      /* The ISR clears the buff_mode variable */
               break;       /* When all of the conversions are complete */
        }
    }
    else
    {
        /* We're in polled mode, so start the first conversion */

        gxad_start_conversion(gxad_input_buffer[gxad_ch_index++]);
        if(gxad_error_code)
		return(-1);
        gxad_wait_ready();
        if(gxad_error_code)
		return(-1);

        /* Read the old data, we don't need it */

        gxad_read_conversion_data();
        if(gxad_error_code)
		return(-1);

        /* While there's channel numbers in the buffer (Not = 0xff)
        convert the requested channel and place the result int the user's
        output buffer.
        */

        while(gxad_input_buffer[gxad_ch_index] != 0xff)
        {
            gxad_start_conversion(gxad_input_buffer[gxad_ch_index++]);
	      if(gxad_error_code)
			return(-1);

            gxad_wait_ready();
            if(gxad_error_code)
			return(-1);

		gxad_user_buffer[gxad_out_index++] = gxad_read_conversion_data();
            if(gxad_error_code)
			return(-1);
        }

        /* One last conversion allow us to retrieve our real last data */

        gxad_start_conversion(0);
        if(gxad_error_code)
		return(-1);

        gxad_wait_ready();
        if(gxad_error_code)
		return(-1);

        gxad_user_buffer[gxad_out_index++] = gxad_read_conversion_data();
        if(gxad_error_code)
		return(-1);

    }

    return 0;

}


//////////////////////////////////////////////////////////////////////////
//
//      GXAD_CONVERT_SINGLE_REPEATED
//
//////////////////////////////////////////////////////////////////////////
int gxad_convert_single_repeated(int channel, unsigned count, unsigned *buffer)
{
int x;

	gxad_error_code = GXAD_SUCCESS;

    /* Setup global variables including transfering the address of the
       user's output buffer to a global variable the ISR knows about.
    */

    gxad_user_buffer = buffer;
    gxad_out_index = 0;
    gxad_repeat_channel = channel;
    gxad_repeat_count = count;

    /* If we're running in interrupt mode, set the flag, and run the
       first conversion manually.
    */

    if(gxad_int_mode)
    {
        gxad_repeat_mode = 1;


        /* Run 1 conversion just to get it started */

        gxad_start_conversion(gxad_repeat_channel);
        if(gxad_error_code)
		return(-1);
        
        /* The ISR should run all of the desired conversions due to repeat
           mode being set.
        */
    
        while(gxad_repeat_mode)     /* This will remain true until the ISR finishes */
        {                      /* all od the desired conversions */
            if(kbhit())
                break;
        }
    }
    else
    {
        /* We're in polled mode. Do the first conversion */

        gxad_start_conversion(gxad_repeat_channel);
        if(gxad_error_code)
		return(-1);

        gxad_wait_ready();
        if(gxad_error_code)
		return(-1);

        /* This data is old, we don't want it */

        gxad_read_conversion_data();
        if(gxad_error_code)
		return(-1);

        /* Perform the requested number of conversions. Place the results
           into the user's output buffer.
        */
        
        for(x=1; x<=gxad_repeat_count; x++)
        {
            gxad_start_conversion(gxad_repeat_channel);
	      if(gxad_error_code)
			return(-1);

            gxad_wait_ready();
	      if(gxad_error_code)
			return(-1);

            gxad_user_buffer[gxad_out_index++] = gxad_read_conversion_data();
	      if(gxad_error_code)
			return(-1);
        }

        /* One last dummy conversion to retrieve our last conversion */

        gxad_start_conversion(gxad_repeat_channel);
        if(gxad_error_code)
		return(-1);

        gxad_wait_ready();
        if(gxad_error_code)
		return(-1);
        gxad_user_buffer[gxad_out_index++] = gxad_read_conversion_data();
        if(gxad_error_code)
		return(-1);

    }

    return 0;
}
//////////////////////////////////////////////////////////////////////////
//
//      GXAD_CONVERT_ALL_CHANNELS
//
/////////////////////////////////////////////////////////////////////////

int gxad_convert_all_channels(unsigned *buffer)
{
int x;

	gxad_error_code = GXAD_SUCCESS;

    /* Initialize global variables including transferring the address of
       the user's output buffer to an internal buffer pointer.
    */

    gxad_user_buffer = buffer;
    gxad_ch_index = 0;
    gxad_out_index = 0;


    /* If we're in interrupt mode set the "buffered" flag so that the
       ISR knows what to do.
    */

    if(gxad_int_mode)
    {
        /* Signal the ISR that he's responsible for starting additional
           conversions according to the input buffer contents.
        */

        gxad_buff_mode = 1;

        /* Run 1 conversion just to get it started */

        gxad_start_conversion(0);
        if(gxad_error_code)
		return(-1);

        /* The ISR should run all of the desired conversions due to buffered
           mode being set.
        */
    
        while(gxad_buff_mode)  /* This remains true until all conversions are complete */
        {
            if(kbhit())
                break;
        }
    }
    else
    {
        /* We're in polled mode start up the first conversion */
 
        gxad_start_conversion(0);
        if(gxad_error_code)
		return(-1);
        gxad_wait_ready();
        if(gxad_error_code)
		return(-1);

        /* This data is old, throw it out */

        gxad_read_conversion_data();
        if(gxad_error_code)
		return(-1);

        /* Finish the rest of the channels */

        for(x=1; x<8; x++)
        {
            gxad_start_conversion(x);
	      if(gxad_error_code)
			return(-1);
            gxad_wait_ready();
	      if(gxad_error_code)
			return(-1);

            /* Store the results in the user's output buffer */

            gxad_user_buffer[gxad_out_index++] = gxad_read_conversion_data();
	      if(gxad_error_code)
			return(-1);
        }

        /* A final dummy conversion is required to get the last of out data */

        gxad_start_conversion(0);
        if(gxad_error_code)
		return(-1);
        gxad_wait_ready();
        if(gxad_error_code)
		return(-1);
        gxad_user_buffer[gxad_out_index++] = gxad_read_conversion_data();
        if(gxad_error_code)
		return(-1);

    }

    return 0;

}

//////////////////////////////////////////////////////////////////////////
//
//      GXAD_INSTALL_ISR
//
//////////////////////////////////////////////////////////////////////////

void gxad_install_isr(int pirq)
{
unsigned vector;
unsigned char mask_val;

    /* Assign the IRQ number to our global variable */

    gxad_irq = pirq;

    /* Calculate the correct vector */

    if(gxad_irq < 8)
    {
        vector = gxad_irq + 8;
        mask_val = 1 << gxad_irq;
    }
    else
    {
        vector = (gxad_irq - 8) + 0x70;
        mask_val = 1 << (gxad_irq - 8);
    }

    /* Get the old contents of the vector and save it away */

    old_isr = getvect(vector);

    /* Put our new vector in its place */

    setvect(vector, gxad_a2d_isr);

    /* Calculate and clear the correct mask bit in the interrupt controller */

    if(gxad_irq < 8)
        outportb(0x21,inportb(0x21) & ~mask_val);
    else
        outportb(0xa1,inportb(0xa1) & ~mask_val);

    /* Now enable interrupts at the LTC185X chip */

    outportb(GXAD_INT_CONTROL,1);
}

//////////////////////////////////////////////////////////////////////////
//
//      GXAD_UNINSTALL_ISR
//
//////////////////////////////////////////////////////////////////////////

void gxad_uninstall_isr(int pirq)
{
unsigned vector;
unsigned char mask_val;

    /* Transfer the Interrupt number to our global variable */

    gxad_irq = pirq;


    /* Now disable interrupts at the LTC185X chip  */

    outportb(GXAD_INT_CONTROL,0);

    /* Calculate and reset the proper mask bit in the interrupt controller */


    if(gxad_irq < 8)
    {
        vector = gxad_irq + 8;
        mask_val = 1 << gxad_irq;
    }
    else
    {
        vector = (gxad_irq - 8) + 0x70;
        mask_val = 1 << (gxad_irq - 8);
    }


    if(gxad_irq < 8)
        outportb(0x21,inportb(0x21) | ~mask_val);
    else
        outportb(0xa1,inportb(0xa1) | ~mask_val);

    /* Put back the original vector contents */

    setvect(vector, old_isr);


}

//////////////////////////////////////////////////////////////////////////
//
//          GXAD_A2D_ISR
//
//////////////////////////////////////////////////////////////////////////
void interrupt gxad_a2d_isr(void)
{
unsigned result;

    /* Set the intflag. This lets the wait_ready routine know the conversion
       is complete.
    */

    ++gxad_intflag;

    /* Read the chip status */

    inportb(GXAD_STATUS_PORT);

    /* Check for buffered mode */

    if(gxad_buff_mode)
    {
        /* Read the data and put it in the buffer */

        result = inportb(GXAD_MSB_DATA);
        result = result << 8;
        result = result | inportb(GXAD_LSB_DATA);

        gxad_user_buffer[gxad_out_index++] = result;

        /* See If there's another channel to start up */

        if(gxad_input_buffer[gxad_ch_index] != 0xff)
            gxad_start_conversion(gxad_input_buffer[gxad_ch_index++]);
        else
            gxad_buff_mode = 0;
    }

    if(gxad_repeat_mode)
    {
        /* Read the data and put it in the buffer  */

        result = inportb(GXAD_MSB_DATA);
        result = result << 8;
        result = result | inportb(GXAD_LSB_DATA);

        gxad_user_buffer[gxad_out_index++] = result;

        if(--gxad_repeat_count == 0)
            gxad_repeat_mode = 0;
        else
            gxad_start_conversion(gxad_repeat_channel);
    }

    /* Output the End of Interrupt (EOI) command to the proper interrupt
       controller(s).
    */

    if(gxad_irq > 8)
        outportb(0xa0,0x20);
    outportb(0x20,0x20);

}


///////////////////////////////////////////////////////////////////////////
//
//           GXAD_SET_CHANNEL_MODE
//
///////////////////////////////////////////////////////////////////////////
int gxad_set_channel_mode(int channel, int input_mode, int duplex, int range)
{
unsigned char command_byte;


	gxad_error_code = GXAD_SUCCESS;

    /* Check for legal channel numbers */

	if((channel < 0) || (channel > 7))
	{
		gxad_error_code = GXAD_BAD_CHANNEL_NUMBER;
		sprintf(gxad_error_string,"GXAD : Set Channel Mode - Bad channel Number %d",channel);
		return(-1);
	}


    /* Check for illegal mode(s) */

	if((input_mode != GXAD_SINGLE_ENDED) && (input_mode != GXAD_DIFFERENTIAL))
	{
		gxad_error_code = GXAD_BAD_MODE_NUMBER;
		sprintf(gxad_error_string,"GXAD : Set Channel Mode - Bad mode Number");
       	return(-1);
	}

	if((duplex != GXAD_UNIPOLAR) && (duplex != GXAD_BIPOLAR))
      {
		gxad_error_code = GXAD_BAD_MODE_NUMBER;
		sprintf(gxad_error_string,"GXAD : Set Channel Mode - Bad mode Number");
       	return(-1);
	}

	if((range != GXAD_TOP_5V) && (range != GXAD_TOP_10V))
	{
		gxad_error_code = GXAD_BAD_RANGE;
		sprintf(gxad_error_string,"GXAD : Set Channel Mode - Bad Range Value");
		return(-1);
	}


    /* Build a command byte out of the arguments */

    command_byte = gxad_channel_select[channel];
    command_byte = command_byte | input_mode | duplex | range;

    gxad_channel_mode[channel] = command_byte;

    /* Calculate bit values, offset, and adjustment values */

    if((range == GXAD_TOP_5V) && (duplex == GXAD_UNIPOLAR))
    {
        gxad_bitval[channel] = 5.000 / 65536.0;
        gxad_adjust[channel] = 0.0;
        gxad_offset[channel] = 0.0;
    }

    if((range == GXAD_TOP_5V) && (duplex == GXAD_BIPOLAR))
    {
        gxad_bitval[channel] = 10.000 / 65536.0;
        gxad_adjust[channel] = 0x8000;
        gxad_offset[channel] = -5.000;
    }

    if((range == GXAD_TOP_10V) && (duplex == GXAD_UNIPOLAR))
    {
        gxad_bitval[channel] = 10.000 / 65536.0;
        gxad_adjust[channel] = 0.0;
        gxad_offset[channel] = 0.0;
    }

    if((range == GXAD_TOP_10V) && (duplex == GXAD_BIPOLAR))
    {
        gxad_bitval[channel] = 20.000 / 65536.0;
        gxad_adjust[channel] = 0x8000;
        gxad_offset[channel] = -10.0;
    }

    return 0;
}
///////////////////////////////////////////////////////////////////////////
//
//      GXAD_SET_INTERRUPT_MODE
//
///////////////////////////////////////////////////////////////////////////
int gxad_set_interrupt_mode(int my_irq)
{
    if(my_irq < 1 || my_irq > 15)
        return(1);
    disable();
    gxad_irq = my_irq;
    gxad_int_mode = 1;
    gxad_install_isr(gxad_irq);
    enable();

    return(0);
}
//////////////////////////////////////////////////////////////////////////
//
//       GXAD_SET_POLLED_MODE
//
//////////////////////////////////////////////////////////////////////////
void gxad_set_polled_mode()
{
    if(!gxad_int_mode)
        return;
    disable();
    gxad_uninstall_isr(gxad_irq);
    gxad_int_mode = 0;
    gxad_irq = 0;
    enable();
}

///////////////////////////////////////////////////////////////////////////
//
//      GXAD_START_CONVERSION
//
//////////////////////////////////////////////////////////////////////////

int gxad_start_conversion(unsigned char channel)
{
	gxad_error_code = GXAD_SUCCESS;
    
	if(channel > 7)
	{
		gxad_error_code = GXAD_BAD_CHANNEL_NUMBER;
		sprintf(gxad_error_string,"GXAD : Start Conversion - Bad channel number %d",channel);
		return(-1);
	}

    /* Set the channel numbers */

    gxad_last_channel = gxad_current_channel;
    gxad_current_channel = channel;

    /* Give the conversion command */

    outportb(GXAD_COMMAND_PORT,gxad_channel_mode[channel]);
    return 0;
}


/////////////////////////////////////////////////////////////////////////
//
//          GXAD_READ_CONVERSION_DATA
//
/////////////////////////////////////////////////////////////////////////

unsigned gxad_read_conversion_data(void)
{
unsigned result;


    /* Read out the two bytes and assemble the word */

    result = inportb(GXAD_MSB_DATA);
    result = result << 8;
    result = result | inportb(GXAD_LSB_DATA);
    return result;
}

/////////////////////////////////////////////////////////////////////////
//
//      GXAD_WAIT_READY
//
//////////////////////////////////////////////////////////////////////////
int gxad_wait_ready(void)
{
unsigned long retry;
unsigned count;

	gxad_error_code = GXAD_SUCCESS;
    count = 0;
    retry = 100000;
    while(retry--)
    {
        if(gxad_int_mode && gxad_intflag)
             return 0;
        count++;
        if(!gxad_int_mode)
        {
            if((inportb(GXAD_STATUS_PORT) & GXAD_BUSY_BIT) == 0)
                return 0;
        }
    }
    if(gxad_int_mode)
    {
	gxad_error_code = GXAD_TIMEOUT_ERROR;
	sprintf(gxad_error_string,"GXAD : Wait Ready - Device timeout error");
	return(-1);
    }
    else
    {
	gxad_error_code = GXAD_TIMEOUT_ERROR;
	sprintf(gxad_error_string,"GXAD : Wait Ready - Device timeout error");
	return(-1);
    }

}

