/* gxa2d.h Copyright 2005 by WinSystems Inc. All Rights Reserved
**********************************************************************
*
*   Name    : gxa2d.h
*
*   Project : EPX-GX User interface code
*
*   Purpose : Analog input routines
*
*   Date    : September 7, 2005
*
*   Revision: 1.00
*
*   Author  : Steve Mottin
*
**********************************************************************
*
*   Revision History
*
*   Date        Revision    Description
*   --------    --------    ----------------------------------------
*   09/07/05    1.00        Initial Version
*
**********************************************************************
*/

/* The Port definitions for the registers */

#define GXAD_BASE_ADDRESS 0x1e8
#define GXAD_COMMAND_PORT GXAD_BASE_ADDRESS
#define GXAD_STATUS_PORT  GXAD_BASE_ADDRESS
#define GXAD_LSB_DATA     GXAD_BASE_ADDRESS+1
#define GXAD_MSB_DATA     GXAD_BASE_ADDRESS+2
#define GXAD_INT_CONTROL  GXAD_BASE_ADDRESS+2

/* These bits appear in the status register */

#define GXAD_INT_BIT     0x80
#define GXAD_BUSY_BIT    0x01

/* These defines make setting the mode easy. The actual register
   bit definitions have no logical order to them
*/

#define GXAD_SINGLE_ENDED 0x80
#define GXAD_DIFFERENTIAL 0x00

#define GXAD_UNIPOLAR    0x08
#define GXAD_BIPOLAR     0x00

#define GXAD_TOP_5V      0x00
#define GXAD_TOP_10V     0x04

/* Even the channel numbers are encoded in the command in a
   strange way in the register.
*/

#define GXAD_CH0_SELECT  0x00
#define GXAD_CH1_SELECT  0x40
#define GXAD_CH2_SELECT  0x10
#define GXAD_CH3_SELECT  0x50
#define GXAD_CH4_SELECT  0x20
#define GXAD_CH5_SELECT  0x60
#define GXAD_CH6_SELECT  0x30
#define GXAD_CH7_SELECT  0x70

/* Here are the function prototypes for all of the functions used in this
   support library.
*/

int gxad_wait_ready(void);
int gxad_start_conversion(unsigned char channel);
int gxad_set_interrupt_mode(int my_irq);
void gxad_set_polled_mode(void);
unsigned gxad_read_conversion_data(void);
int gxad_set_channel_mode(int channel, int input_mode, int duplex, int range);
int gxad_convert_all_channels(unsigned *buffer);
int gxad_convert_single_repeated(int channel, unsigned count, unsigned *buffer);
int gxad_buffered_channel_conversions(unsigned char *input_channel_buffer,unsigned *buffer);
float gxad_get_channel_voltage(int channel);
float gxad_auto_get_channel_voltage(int channel);
void gxad_install_isr(int irq);
void gxad_uninstall_isr(int irq);
void interrupt gxad_a2d_isr(void);

/* To make channel selection much, much easier (no shifting and oring)
   we build a select table array the channel number can be used as the
   index into the array and the proper bit encoding for that channel id
   the result.
*/
extern unsigned char gxad_channel_select[8];

/* Along the same line when selecting a channel, the mode bits are also
   pre-encoded using this array. It starts out blank and is filled in
   when making calls to set_channel_mode.
*/

extern unsigned char gxad_channel_mode[8];

/* These three arrays are used primarily for conversion to floating point
   voltage values. THey are also filled in for a channel during a call to
   set_channel_mode. They avoid having to bit decode command bytes to
   figure out what mode is being used and therefore the voltage value of
   each bit.
*/

extern float gxad_bitval[8];
extern unsigned gxad_adjust[8];
extern float gxad_offset[8];


/* A bunch of global variables are used to make processing faster and to
   flag the ISR (when in interrupt mode) as to proper handling of the
   data and whether to start up another conversion from within the ISR
   itself.
*/

/* Int_mode is the switch thet controlls whether the code will run interrupt
   driven or in a polled mode. When int_mode is 0, polled I/O will be used,
   when non-zero interrupt I/O will be used. Because of the extreme
   speed of the LTC185X converters, interrupt mode is about 10% slower in
   most cases due to interrupt overhead.
*/

extern int gxad_int_mode;
extern int gxad_irq;
extern int gxad_intflag;
extern int gxad_buff_mode;

/*  These two variables can be useful because when a conversion is
    started the data from the previous conversion is returned. These
    variables can be used to determine which channel's data is being
    returned by read_conversion_data wich will return the last_channel's
    data. Note that the get_voltage and auto_get_voltage autmatically do
    double reads in order to return the true currently requested channel
    voltage.
*/

extern int gxad_last_channel;
extern int gxad_current_channel;

/* These are used to control repeated single channel conversions at
   in a high-speed buffered manner.
*/

extern int gxad_repeat_mode;
extern int gxad_repeat_count;
extern int gxad_repeat_channel;
extern int gxad_repeat;

/* Storage for the previous contents of the specified interrupt vector.
   It is fillled in with a call to install_isr and is used by uninstall_isr
   to return the vector to its previous state.
*/

extern void interrupt (*old_isr)();

/* An array of channel numbers is used by the convert_all_channels function
   and the others are index pointers into user supplied buffers used
   during buffered functions.
*/

extern unsigned char gxad_channel_buff[10];
extern unsigned char *gxad_input_buffer;
extern unsigned *gxad_user_buffer;
extern int gxad_ch_index;
extern int gxad_out_index;

extern int gxad_error_code;
extern char gxad_error_string[];

#define GXAD_SUCCESS 0
#define GXAD_OPEN_ERROR 1
#define GXAD_TIMEOUT_ERROR 2
#define GXAD_BAD_CHANNEL_NUMBER 3
#define GXAD_BAD_MODE_NUMBER 4
#define GXAD_BAD_RANGE 5
#define GXAD_COMMAND_WRITE_FAILURE 6
#define GXAD_READ_DATA_FAILURE 7

